from coopgame.gameTemplate import GameTemplate
from coopgame import handleKeyPressedArgs as hkpa
from coopgame import pygamehelpers as pgh
from coopgame.label_drawing import label_drawing_utils as ldu
from coopgame.pointdrawing import point_draw_utils as pdu
from cooptools.colors import Color
from coopstructs.geometry import Rectangle
from cooptools.coopEnum import CardinalPosition
from cooptools.toggles import IntegerRangeToggleable
import math

SCREEN_WIDTH = 1000
SCREEN_HEIGHT = 750

class RectDraw(GameTemplate):
    def __init__(self):

        self.rot_toggle = IntegerRangeToggleable(min=0, max=100, step_size=1, starting_value=0)

        input_state_handler = hkpa.InputStateHandler(key_actions=self.default_key_actions() + [
            hkpa.InputAction([hkpa.InputEvent(hkpa.PyKeys.K_LEFT,
                                              event_type=hkpa.InputEventType.HELD)],
                             callback=lambda x: self.rot_toggle.toggle(loop=True)),
            hkpa.InputAction([hkpa.InputEvent(hkpa.PyKeys.K_RIGHT,
                                              event_type=hkpa.InputEventType.HELD)],
                             callback=lambda x: self.rot_toggle.toggle(loop=True, reverse=True))
        ])



        super().__init__(input_state_handler=input_state_handler,
                         screen_width=SCREEN_WIDTH,
                         screen_height=SCREEN_HEIGHT)

    def draw_waypoints(self):
        points = {}
        for id, bg in self._model.BattleGroups.items():
            points = {**points,
                **{x.end_pos: pdu.DrawPointArgs(
                    color=Color.ORANGE,
                    radius=3
                )
                for x in bg.waypoint_follower.waypoints}
            }

            if len(bg.waypoint_follower.waypoints) > 0:
                pgh.draw_arrow(
                    self.screen,
                    start=bg.Transform.Translation.Vector,
                    end=bg.waypoint_follower.waypoints[0].end_pos,
                    color=Color.GRAY,
                    leader_line=True
                )

        pdu.draw_points(
            points=points,
            surface=self.screen
        )

    def _get_rot(self):
        return self.rot_toggle.value / 100 * 2 * math.pi

    def draw(self, frames: int, debug_mode: bool = False):
        rect = Rectangle.from_tuple((
            SCREEN_WIDTH // 2,
            SCREEN_HEIGHT // 2,
            50,
            25),
            anchor_cardinality=CardinalPosition.CENTER,
            rotation_rads=self._get_rot()
        )
        print(rect.RotationRads)
        pgh.draw_box(
            surface=self.screen,
            rect=rect,
            color=Color.BLUE,
            outline_color=Color.RED,
            anchor_draw_args=pgh.DrawPointArgs(
                color=Color.ORANGE,
            ),
            corner_draw_args=pgh.DrawPointArgs(
                color=Color.YELLOW,
                radius=3
            )
        )


if __name__ =="__main__":
    import logging
    from cooptools.loggingHelpers import BASE_LOG_FORMAT
    logging.basicConfig(level=logging.INFO, format=BASE_LOG_FORMAT)
    logging.getLogger('cooptools.physics.kinematic').setLevel(logging.WARN)
    logging.getLogger('cooptools.sectors.sectorTree.sectorTree_SectorComparer').setLevel(logging.WARN)
    game = RectDraw()
    game.main()




import math
from typing import List

import pygame.draw
from coopstructs.geometry.vectors.vectorN import Vector2
from coopgame.gameTemplate import GameTemplate, BuiltInSurfaceType
from coopgame.particleSystem import Particle, Emitter, particle_destroyer, time_destroyer, InterpolateColorBehavior, ParticleGenerationArgs
import coopgame.particleSystem.recipies as recip
import random as rnd
from cooptools.colors import Color
from functools import partial
from cooptools.dataRefresher import DataRefresher
import logging
from coopgame.logger import logger
from coopgame.particleSystem import ParticleRenderer, RenderShape, PulseColorBehavior, LifetimeType
import coopgame.particleSystem.utils as utils

class ParticleGame(GameTemplate):


    def initialize_game(self):
        self.firework_launch_sound = pygame.mixer.Sound(
            "../coopgame/assets/sounds/81455__csengeri__fireworks-launch-denoised.wav")
        self.firework_explosion_sound = pygame.mixer.Sound(
            "../coopgame/assets/sounds/553153__eminyildirim__firework-single-explosion.wav")

        snow_renderer = ParticleRenderer(
            shape=RenderShape.CIRCLE,
            colorBehavior=Color.SNOW
        )
        self.snow = recip.snow(
            origin_line_start=Vector2(0,0),
            origin_line_end=Vector2(self.screen_width, 0),
            fall_velo=Vector2(0, 100),
            lifespan_ms=15000,
            size=3,
            twinkle_ms=500,
            renderer=snow_renderer,
            particle_generation_args=ParticleGenerationArgs(generation_interval_ms=100)
        )

        comet_lifespan = 1000
        comet_colors = [
                    Color.WHITE,
                    Color.PURPLE,
                    Color.ORANGE,
                    Color.BLUE,

                ]
        comet_renderer = ParticleRenderer(
            shape=RenderShape.CIRCLE,
            colorBehavior=PulseColorBehavior(
                colors=comet_colors,
                pulse_duration_ms=int(comet_lifespan/len(comet_colors))
            ),
            lifetime_type=LifetimeType.EMITTER
        )
        self.comet = recip.comet(
            path=lambda: utils.rotation(self.game_time_tracker.run_time, self.screen_height, self.screen_width),
            radiate_speed=50,
            lifespan_ms=comet_lifespan,
            start_size=5,
            renderer=comet_renderer,
            particle_generation_args=ParticleGenerationArgs(generation_interval_ms=5)
        )

        comet_lifespan = 1000
        comet_colors = [
                    Color.WHITE,
                    Color.PURPLE,
                    Color.ORANGE,
                    Color.BLUE,

                ]
        comet_renderer = ParticleRenderer(
            shape=RenderShape.CIRCLE,
            colorBehavior=PulseColorBehavior(
                colors=comet_colors,
                pulse_duration_ms=int(comet_lifespan/len(comet_colors))
            ),
            lifetime_type=LifetimeType.PARTICLE
        )
        self.comet2 = recip.comet(
            path=lambda: utils.rotation(self.game_time_tracker.run_time, self.screen_height, self.screen_width, phase=500),
            radiate_speed=50,
            lifespan_ms=comet_lifespan,
            start_size=5,
            renderer=comet_renderer,
            particle_generation_args=ParticleGenerationArgs(generation_interval_ms=5)
        )

        fire_renderer = ParticleRenderer(
            colorBehavior=InterpolateColorBehavior(start_color=Color.ORANGE, end_color=Color.PURPLE, lifespan_ms=1500)
        )
        smoke_renderer = ParticleRenderer(
            colorBehavior=InterpolateColorBehavior(start_color=Color.DARK_GRAY, end_color=Color.BLACK, lifespan_ms=5000)
        )
        self.torch = recip.torch(
            Vector2(250, 250),
            fire_lifespan_ms=1500,
            smoke_lifespan_ms=5000,
            fire_start_size=10,
            smoke_end_size=20,
            fire_raise_velo=Vector2(0, -50),
            smoke_raise_velo=Vector2(0, -15),
            fire_renderer=fire_renderer,
            smoke_renderer=smoke_renderer
        )

        fountain_particle_lifespan_ms = 2750
        fountain_renderer = ParticleRenderer(
            colorBehavior=InterpolateColorBehavior(start_color=Color.DARK_BLUE, end_color=Color.ALICE_BLUE,
                                                   lifespan_ms=fountain_particle_lifespan_ms),
            reverse=True
        )
        self.fountain = recip.fountain(
            origin=Vector2(500, 250),
            spray_intensity=150,
            spray_arc=(75, 105),
            fountain_lifespan_ms=fountain_particle_lifespan_ms,
            drop_accel=Vector2(0, 100),
            droplet_size_start=10,
            droplet_size_end=1,
            droplet_renderer=fountain_renderer,
            droplet_generation_args=ParticleGenerationArgs(generation_interval_ms=10)
        )


        f_origin = lambda: Vector2(rnd.randint(0, self.screen_width), self.screen_height)
        firework_renderer = ParticleRenderer(
            colorBehavior=InterpolateColorBehavior(start_color=lambda: Color.random(), end_color=Color.BLACK,
                                                   lifespan_ms=fountain_particle_lifespan_ms),
        )
        self.firework = recip.firework(
            origin=f_origin,
            launch_speed=lambda : rnd.randint(200, 250),
            spread=lambda: rnd.randint(650, 850),
            lifespan_ms=rnd.randint(500, 750),
            drop_accel=Vector2(0, 20),
            size_start=5,
            size_end=1,
            arms=lambda: rnd.randint(3, 20),
            explosion_renderer=firework_renderer,
            launch_renderer=ParticleRenderer(colorBehavior=Color.WHITE),
            burst_interval_ms=lambda: rnd.randint(500, 2500),
            launch_start_callback=lambda x: self.firework_launch_sound.play(),
            launch_end_callback = lambda x: self.firework_launch_sound.stop(),
            explosion_start_callback= lambda x: self.firework_explosion_sound.play(),
            explosion_end_callback=lambda x: self.firework_explosion_sound.stop(),

        )


        self.log_part_refresher = DataRefresher('log_stats',
                                                 refresh_callback=lambda: self.particle_stats(),
                                                 refresh_interval_ms=5000)

    def handle_left_mouse_down(self, pressed_keys, mouse_pos_as_vector: Vector2):
        pass

    def handle_left_mouse_up(self, pressed_keys, mouse_pos_as_vector: Vector2):
        pass

    def handle_right_mouse_down(self, pressed_keys, mouse_pos_as_vector: Vector2):
        pass

    def handle_right_mouse_up(self, pressed_keys, mouse_pos_as_vector: Vector2):
        pass

    def handle_hover_over(self, pressed_keys, mouse_pos_as_vector: Vector2):
        pass

    def handle_mouse_scroll_up(self, pressed_keys, mouse_pos_as_vector: Vector2):
        pass

    def handle_mouse_scroll_down(self, pressed_keys, mouse_pos_as_vector: Vector2):
        pass

    def draw(self, frames: int, debug_mode: bool = False):
        self.snow.render(self.screen)
        self.comet.render(self.screen)
        self.comet2.render(self.screen)
        self.torch.render(self.screen)
        self.fountain.render(self.screen)
        self.firework.render(self.screen)

    def model_updater(self, delta_time_ms: int):
        self.snow.update(delta_time_ms)
        self.comet.update(delta_time_ms)
        self.comet2.update(delta_time_ms)
        self.torch.update(delta_time_ms)
        self.fountain.update(delta_time_ms)
        self.firework.update(delta_time_ms)

        self.log_part_refresher.check_and_refresh()

    def sprite_updater(self, delta_time_ms: int):
        pass

    def on_resize(self):
        pass

    def register_keys(self):
        pass

    def update_built_in_surfaces(self, surface_types: List[BuiltInSurfaceType]):
        pass

    def particle_stats(self):
        logger.info(f"Particle Count: \n"
                    f"Snow: {self.snow.n_particles()}\n"
                     f"Comet: {self.comet.n_particles()}\n"
                     f"Torch: {self.torch.n_particles()}\n"
                    f"Fountain: {self.fountain.n_particles()}\n"
                    f"Firework: {self.firework.n_particles()}")

if __name__ == "__main__":
    game = ParticleGame()
    game.main()


import pygame
import numpy as np
from typing import Tuple
from cooptools.colors import Color
import coopgame.pygamehelpers as help
from coopstructs.geometry.rectangles.rectangle import Rectangle
from typing import List
import cooptools.geometry_utils.vector_utils as vec

"""
This implemenation is based off of the description on the pygame website docs at: https://pygame.readthedocs.io/en/latest/tiles/tiles.html
"""
class TileSet:
    def __init__(self, tiles: List[pygame.Surface] = None):
        self.tiles = tiles or []

class FileTileset(TileSet):
    def __init__(self, file, size=(32, 32), margin=1, spacing=1):
        super().__init__()
        self.file = file
        self.size = size
        self.margin = margin
        self.spacing = spacing
        self.image = pygame.image.load(file)
        self.rect = self.image.get_rect()
        self.load()

    def load(self):
        self.tiles = []
        x0 = y0 = self.margin
        w, h = self.rect.size
        dx = self.size[0] + self.spacing
        dy = self.size[1] + self.spacing

        for x in range(x0, w, dx):
            for y in range(y0, h, dy):
                tile = pygame.Surface(self.size)
                tile.blit(self.image, (0, 0), (x, y, *self.size))
                self.tiles.append(tile)

    def __str__(self):
        return f'{self.__class__.__name__} file:{self.file} tile:{self.size}'




class Tilemap:
    def __init__(self,
                 tileset: TileSet,
                 map_size=None,
                 render_on_init: bool = True,
                 surface_size: Tuple[int, int] = None,
                 buffer_tiles: Tuple[int, int] = None,
                 map: np.ndarray = None,
                 pxls: int = 32,
                 # outline_tile_color: Color = None
                 ):

        self.pxls = pxls
        self._tile_size = (self.pxls, self.pxls)

        if map is not None:
            self.map = map
        elif map_size:
            self.map = np.zeros(map_size, dtype=int)
        else:
            self.map = np.zeros((10, 20), dtype=int)

        self.size = self.map.shape
        self.buffer_tiles = buffer_tiles

        self.tileset = tileset
        self.image = None

        self.render_surface_size = surface_size
        if buffer_tiles is not None:
            self.render_surface_size = (self.render_surface_size[0] + self.buffer_tiles[0] * self._tile_size[0],
                                        self.render_surface_size[1] + self.buffer_tiles[1] * self._tile_size[1])

        self._valid = False

        if render_on_init:
            self.render(self.render_surface_size)
            self._valid = True

    def render(self, surface_size: Tuple[int, int], outline_tiles_color: Color = None):
        h, w = self.map.shape

        self.render_surface_size = surface_size
        self.image = pygame.Surface(surface_size)
        self.image.fill(Color.HOT_PINK.value)
        self.image.set_colorkey(Color.HOT_PINK.value)
        self.image.convert_alpha()

        tile_size_w = surface_size[0] / w
        tile_size_h = surface_size[1] / h
        self._tile_size = (tile_size_w, tile_size_h)

        for i in range(w):

            for j in range(h):
                tile = self.tileset.tiles[self.map[j, i]]
                scaled_tile = pygame.transform.scale(tile, (int(tile_size_w), int(tile_size_h)))
                self.image.blit(scaled_tile, (i*tile_size_w,j*tile_size_h))
                if outline_tiles_color is not None:
                    help.draw_box(self.image,
                                  Rectangle.from_meta(x=i * tile_size_w,
                                            y=j * tile_size_h,
                                            width=tile_size_w,
                                            height=tile_size_h),
                                  color=outline_tiles_color,
                                  width=1)

    def draw(self, surf: pygame.Surface, outline_tiles_color: Color = None):
        # if self._valid is False:
        self.render(surface_size=surf.get_size(), outline_tiles_color=outline_tiles_color)

        surf.blit(self.image, self.rect)

    def invalidate(self):
        self._valid = False

    def set_value(self, value, positions: vec.IterVec = None):
        if positions is not None:
            for pos in positions:
                self.map[(int(pos[0]), int(pos[1]))] = value
        else:
            self.map.fill(value)

    def _increment_at_pos(self, pos, increment):
        new_val = self.map[(int(pos[0]), int(pos[1]))] + increment
        if new_val >= len(self.tileset.tiles):
            new_val = 0
        if new_val < 0:
            new_val = len(self.tileset.tiles) - 1
        self.map[(int(pos[0]), int(pos[1]))] = new_val


    def increment_value(self, positions: vec.IterVec = None, increment: int = 1):
        if positions is not None:
            for pos in positions:
                self._increment_at_pos(pos, increment)
        else:
            for r in self.map.shape[0]:
                for c in self.map.shape[1]:
                    self._increment_at_pos((r,c), increment)

    def set_random(self):
        n = len(self.tileset.tiles)
        self.map = np.random.randint(n, size=self.size)

        # self.invalidate()
        # self.render(self.render_surface_size, outline_tiles_color=self.outline_tile_color)

    def __str__(self):
        return f'{self.__class__.__name__} {self.size}'

    @property
    def rect(self):
        return self.image.get_rect()

    @property
    def TileSize(self):
        return self._tile_size